/*
    Copyright (C) 1999 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ardour_ui2.cc,v 1.41 2000/11/03 19:45:39 pbd Exp $
*/

#include <algobase.h>

#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#include <iostream>

#include <gtk--.h>
#include <pbd/stl_delete.h>
#include <pbd/error.h>
#include <pbd/basename.h>
#include <gtkmmext/pix.h>
#include <gtkmmext/pixmap_button.h>
#include <gtkmmext/utils.h>
#include <gtkmmext/led.h>
#include <gtkmmext/click_box.h>

#include "ardour.h"
#include "ardour_ui.h"
#include "track_ui.h"
#include "audio_clock_ui.h"
#include "alsa_device.h"
#include "location_ui.h"

using namespace ARDOUR;
using namespace Gtkmmext;
using namespace Gtk;
using namespace SigC;

#define FRAME_SHADOW_STYLE GTK_SHADOW_IN
#define FRAME_NAME "BaseFrame"

static int
time_to_die (GdkEventAny *ev)

{
	gdk_key_repeat_restore ();
	/* not very C++, but it avoids making theMain global */
	gtk_main_quit ();
	return TRUE;
}

int	
ARDOUR_UI::setup_windows (int argc, char **argv)

{
	using namespace Menu_Helpers;

	if (setup_pixmaps ()) {
		return -1;
	}

	if (setup_wipe_window ()) {
		return -1;
	}

	build_menu_bar ();

	location_window.set_title ("ardour: locations");
	location_window.delete_event.connect (slot(time_to_die)); /* XXXX fix me */
	
	control_window.set_title ("ardour: controls");
	control_window.delete_event.connect (slot(time_to_die)); /* XXXX fix me */

	track_vbox.pack_start (upper_track_box, false, false, 10);
	track_vbox.pack_start (meter_frame, false, false, 5);
	track_vbox.pack_start (lower_track_box, false, false, 5);

	upper_track_box.set_name ("AboveMeterZone");
	lower_track_box.set_name ("BelowMeterZone");

	meter_base.set_name ("MeterBase");
	meter_frame.set_shadow_type (FRAME_SHADOW_STYLE);
	meter_frame.set_name (FRAME_NAME);
	meter_frame.add (meter_base);

	setup_tape_menu ();
	setup_transport();
	setup_clock ();
	setup_logo ();
	setup_tape_info ();
	setup_adjustables ();
	setup_button_table ();

	/* Location UI */

	location_ui = new Location_UI (mcd, main_window);

	location_frame.add (*location_ui);
	location_frame.set_shadow_type (FRAME_SHADOW_STYLE);
	location_frame.set_name (FRAME_NAME);
	location_hpacker.pack_start (location_frame, false, false, 5);
	location_vpacker.pack_start (location_hpacker, false, false, 5);
	location_base.put (location_vpacker, 0, 0);
	location_base.set_name ("TransportWindow");
	location_window.add (location_base);

	ctl_hpacker.pack_start (button_frame, false, false, 10);
	ctl_vpacker.pack_start (ctl_hpacker, true, true, 10);

	ctl_base.put (ctl_vpacker, 0, 0);
	ctl_base.set_name ("ControlWindow");
	control_window.add (ctl_base);

	/* Now do the main window */

	recorder_base.set_name ("RecorderWindow");
	recorder_base.put (recorder_vpacker, 0, 0);

	notebook.set_show_tabs (false);
	notebook.switch_page.connect (slot (this, &ARDOUR_UI::notebook_switched));
	notebook.pages().push_back (Gtk::Notebook_Helpers::TabElem (recorder_base, "Recorder"));

	if (create_editor ()) {
		error << "UI: cannot setup editor"
		      << endmsg;
		return -1;
	}

	upper_hpacker.pack_start (track_vbox, false, false, 10);

	recorder_vpacker.pack_start (menu_bar, false, false);
	recorder_vpacker.pack_start (upper_hpacker, false, false, 2);
	recorder_vpacker.pack_start (logo_vpacker, false, false, 2);

	build_track_uis ();

	tc_hpacker.set_spacing (5);
	tc_hpacker.set_border_width (5);
	tc_hpacker.pack_start (clock_frame, false, false);
	tc_hpacker.pack_end (transport_frame, false, false);

	tc_vpacker.pack_start (tc_hpacker, false, false, 5);

	tc_base.put (tc_vpacker, 0, 0);
	tc_base.set_name ("RecorderWindow");
	
	main_window_vpacker.pack_start (notebook, true, true);
	main_window_vpacker.pack_start (tc_base, false, false);

	main_window.set_title ("ardour 32/96 multichannel disk recorder");
	main_window.delete_event.connect (slot(time_to_die));	
	main_window.add (main_window_vpacker);
	main_window.show_all ();

	/* Install a top level keypress handler */

	main_window.key_press_event.connect 
		(slot (this, &ARDOUR_UI::global_key_press_handler));
	main_window.key_release_event.connect 
		(slot (this, &ARDOUR_UI::global_key_release_handler));

	return 0;
}

void
ARDOUR_UI::setup_button_table ()

{
	/* Button Array */

	clock_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	clock_button_label.set_text ("CLOCK");
	clock_button_label.set_name ("GlobalButtonLabel");
	
	global_meter_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	global_meter_button_label.set_text ("METERS");
	global_meter_button_label.set_name ("GlobalButtonLabel");

	reset_meters_button = new PixmapButton
		(ARDOUR_UI::instance()->push_button_pix(), 
		 "GlobalButton", false);
	reset_meters_button_label.set_text ("RST METERS");
	reset_meters_button_label.set_name ("GlobalButtonLabel");
	
	rehearse_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	rehearse_button_label.set_text ("REHEARSE");
	rehearse_button_label.set_name ("GlobalButtonLabel");
	

	all_safe_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	all_safe_button_label.set_text ("ALL SAFE");
	all_safe_button_label.set_name ("GlobalButtonLabel");

	single_button_record_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	single_button_record_button_label.set_text ("REC ONE");
	single_button_record_button_label.set_name ("GlobalButtonLabel");
	
	dynamic_punch_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	dynamic_punch_button_label.set_text ("DYN PUNCH");
	dynamic_punch_button_label.set_name ("GlobalButtonLabel");

	auto_punch_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(),
		 "GlobalButton", false);
	auto_punch_button_label.set_text ("AUTO PUNCH");
	auto_punch_button_label.set_name ("GlobalButtonLabel");

	auto_loop_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	auto_loop_button_label.set_text ("AUTO LOOP");
	auto_loop_button_label.set_name ("GlobalButtonLabel");

	auto_play_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	auto_play_button_label.set_text ("AUTO PLAY");
	auto_play_button_label.set_name ("GlobalButtonLabel");

	auto_input_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	auto_input_button_label.set_text ("AUTO INPUT");
	auto_input_button_label.set_name ("GlobalButtonLabel");

	nstring pixmap_path;

	if (mcd->has_hw_monitoring()) {
		hw_monitor_button = new PixmapButton
			(ARDOUR_UI::instance()->toggle_button_pix(),
			 "GlobalButton", false);
		hw_monitor_button_label.set_text ("HW MON");
		hw_monitor_button_label.set_name ("GlobalButtonLabel");
	} else {
		hw_monitor_button = 0;
	}

	bypass_button = new PixmapButton
		(ARDOUR_UI::instance()->toggle_button_pix(), 
		 "GlobalButton", false);
	bypass_button_label.set_text ("BYPASS");
	bypass_button_label.set_name ("GlobalButtonLabel");
	
	save_button = new PixmapButton
		(ARDOUR_UI::instance()->push_button_pix(),
		 "LocationSelectButton", false);
	restore_button = new PixmapButton 
		(ARDOUR_UI::instance()->push_button_pix(),
		 "LocationSelectButton", false);
	save_button_label.set_text("SAVE");
	save_button_label.set_name ("GlobalButtonLabel");
	restore_button_label.set_text ("RESTORE");
	restore_button_label.set_name ("GlobalButtonLabel");

	/* button layout */

//	button_table.set_homogeneous (true);
	button_table.set_col_spacings (5);

	button_table.attach (clock_button_label, 0, 1, 
			     0, 1, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*clock_button, 0, 1, 
			     1, 2, GTK_EXPAND, 0, 0, 0);
	button_table.attach (global_meter_button_label, 1, 2, 
			     0, 1, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*global_meter_button, 1, 2, 
			     1, 2, GTK_EXPAND, 0, 0, 0);
	button_table.attach (reset_meters_button_label, 2, 3, 
			     0, 1, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*reset_meters_button, 2, 3, 
			     1, 2, GTK_EXPAND, 0, 0, 0);

	button_table.attach (rehearse_button_label, 0, 1, 
			     2, 3, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*rehearse_button, 0, 1, 
			     3, 4, GTK_EXPAND, 0, 0, 0);
	button_table.attach (single_button_record_button_label, 1, 2, 
			     2, 3, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*single_button_record_button, 1, 2, 
			     3, 4, GTK_EXPAND, 0, 0, 0);
	button_table.attach (all_safe_button_label, 2, 3, 
			     2, 3, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*all_safe_button, 2, 3, 
			     3, 4, GTK_EXPAND, 0, 0, 0);

	button_table.attach (dynamic_punch_button_label, 0, 1, 
			     4, 5, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*dynamic_punch_button, 0, 1, 
			     5, 6, GTK_EXPAND, 0, 0, 0);
	button_table.attach (auto_punch_button_label, 1, 2, 
			     4, 5, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*auto_punch_button, 1, 2, 
			     5, 6, GTK_EXPAND, 0, 0, 0);
	button_table.attach (auto_loop_button_label, 2, 3, 
			     4, 5, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*auto_loop_button, 2, 3, 
			     5, 6, GTK_EXPAND, 0, 0, 0);

	button_table.attach (auto_play_button_label, 0, 1, 
			     6, 7, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*auto_play_button, 0, 1, 
			     7, 8, GTK_EXPAND, 0, 0, 0);
	button_table.attach (auto_input_button_label, 1, 2, 
			     6, 7, GTK_EXPAND, 0, 0, 0);
	button_table.attach (*auto_input_button, 1, 2, 
			     7, 8, GTK_EXPAND, 0, 0, 0);

	/* SPACE LEFT IN THIS ROW ABOVE */

	if (hw_monitor_button) {
		button_table.attach (hw_monitor_button_label, 0, 1, 
				     11, 12, GTK_EXPAND, 0, 0, 0);	
		button_table.attach (*hw_monitor_button, 0, 1, 
				     12, 13, GTK_EXPAND, 0, 0, 0);
	}

	button_table.attach (bypass_button_label, 1, 2, 
			     11, 12, GTK_EXPAND, 0, 0, 0);	
	button_table.attach (*bypass_button, 1, 2, 
			     12, 13, GTK_EXPAND, 0, 0, 0);

	button_table.attach (save_button_label, 0, 1, 
			     13, 14, GTK_EXPAND, 0, 0, 0);	
	button_table.attach (*save_button, 0, 1, 
			     14, 15, GTK_EXPAND, 0, 0, 0);
	button_table.attach (restore_button_label, 1, 2, 
			     13, 14, GTK_EXPAND, 0, 0, 0);	
	button_table.attach (*restore_button, 1, 2, 
			     14, 15, GTK_EXPAND, 0, 0, 0);

	clock_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_clock));
	global_meter_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_metering));
	reset_meters_button->clicked.connect
		(slot(this,&ARDOUR_UI::reset_meters));
	
	dynamic_punch_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_dynamic_punch));
	auto_punch_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_auto_punch));
	auto_loop_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_auto_loop));
	
	rehearse_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_rehearse));
	
	auto_play_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_auto_play));
	auto_input_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_auto_input));
	bypass_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_bypass));
	
	single_button_record_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_single_button_record));
	rehearse_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_rehearse));
	all_safe_button->clicked.connect
		(slot(this,&ARDOUR_UI::toggle_all_safe));

	if (hw_monitor_button) {
		hw_monitor_button->clicked.connect
			(slot (this,&ARDOUR_UI::toggle_hw_monitoring));
	}

	save_button->clicked.connect (slot (this, &ARDOUR_UI::save_state));
	restore_button->clicked.connect (slot (this, &ARDOUR_UI::restore_state));

	button_hbox.pack_start (button_table, false, false, 10);
	button_vbox.pack_start (button_hbox, false, false, 10);
	button_vbox.pack_start (adjuster_table, false, false, 10);

	button_base.put (button_vbox, 0, 0);
	button_base.set_name ("ButtonTable");
	button_frame.add (button_base);
	button_frame.set_shadow_type (FRAME_SHADOW_STYLE);
	button_frame.set_name (FRAME_NAME);
}


void
ARDOUR_UI::setup_logo ()

{
	nstring pixmap_path;

	pixmap_path = gtk_rc_find_pixmap_in_path (0, "penguin.xpm");
	penguin_pixmap = new Gtk::Pixmap (pixmap_path);

	logo_label1.set_text ("ARDOUR 32/96 ");
	logo_label2.set_text ("linux multichannel disk recorder ");
		
	logo_label1.set_name ("PrimaryLogo");
	logo_label2.set_name ("SecondaryLogo");
	
	logo_hpacker.pack_start (*penguin_pixmap, false, false, 10);
	logo_hpacker.pack_start (logo_label1, false, false);
	logo_hpacker.pack_start (tape_id_vbox, true, true, 30);
	logo_hpacker.pack_end (logo_label2, false, false, 10);

	hsep1_box.pack_start (hsep1, true, true, 15);
	hsep1.set_name ("PrimaryLogo");

	hsep2.set_name ("PrimaryLogo");
	hsep2_box.pack_start (hsep2, true, true, 15);

	logo_vpacker.pack_start (hsep1_box, false, false);
	logo_vpacker.pack_start (logo_hpacker, false, false);
	logo_vpacker.pack_start (hsep2_box, false, false);
}
void
ARDOUR_UI::setup_tape_info ()

{
	tape_id_display.set_name ("TapeID");
	tape_id_display.set_editable (false);

	/* use descender & ascender to make sure its big enough */
	
	set_usize_to_display_given_text 
		(tape_id_display, "ghmmmmmmmmmmmmmmmmmmmmmm",
		 10, 10);
	tape_id_frame.set_shadow_type (FRAME_SHADOW_STYLE);
	tape_id_frame.set_name (FRAME_NAME);
	tape_id_frame.add (tape_id_display);
	tape_id_vbox.pack_start (tape_id_frame, false, false, 5);
}

void
ARDOUR_UI::setup_adjustables ()

{
	adjuster_table.set_homogeneous (true);

	xfade_button = new ClickBox (&xfade_adjustment, "ClickButton");
	xfade_frame.add (*xfade_button);
	xfade_frame.set_name (FRAME_NAME);
	xfade_frame.set_shadow_type (GTK_SHADOW_IN);
	xfade_button->set_sensitive (false);
	xfade_label.set_text ("CROSSFADE");
	xfade_label.set_name ("GlobalButtonLabel");
	xfade_vbox.pack_start (xfade_label, false, false);
	xfade_vbox.pack_start (xfade_frame, false, false, 5);

	xfade_adjustment.value_changed.connect(slot(this,&ARDOUR_UI::set_xfade));

	preroll_button = new ClickBox (&preroll_adjustment, "ClickButton");
	preroll_frame.add (*preroll_button);
	preroll_frame.set_name (FRAME_NAME);
	preroll_frame.set_shadow_type (GTK_SHADOW_IN);
	preroll_label.set_text ("PREROLL");
	preroll_label.set_name ("GlobalButtonLabel");
	preroll_vbox.pack_start (preroll_label, false, false);
	preroll_vbox.pack_start (preroll_frame, false, false, 5);

	postroll_button = new ClickBox (&postroll_adjustment, "ClickButton");
	postroll_frame.add (*postroll_button);
	postroll_frame.set_name (FRAME_NAME);
	postroll_frame.set_shadow_type (GTK_SHADOW_IN);
	postroll_label.set_text ("POSTROLL");
	postroll_label.set_name ("GlobalButtonLabel");
	postroll_vbox.pack_start (postroll_label, false, false);
	postroll_vbox.pack_start (postroll_frame, false, false, 5);

	online_control_strings.push_back ("MMC + Local");
	online_control_strings.push_back ("MMC");
	online_control_strings.push_back ("Local");

	online_control_button = new GlobalClickBox ("CONTROL",
						    online_control_strings);

	online_control_button->adjustment.value_changed.connect(slot(this,&ARDOUR_UI::control_methods_adjusted));

	mmc_id_strings.push_back ("1");
	mmc_id_strings.push_back ("2");
	mmc_id_strings.push_back ("3");
	mmc_id_strings.push_back ("4");
	mmc_id_strings.push_back ("5");
	mmc_id_strings.push_back ("6");
	mmc_id_strings.push_back ("7");
	mmc_id_strings.push_back ("8");
	mmc_id_strings.push_back ("9");

	mmc_id_button = new GlobalClickBox ("MMC ID", mmc_id_strings);

	mmc_id_button->adjustment.value_changed.connect
		(slot(this,&ARDOUR_UI::mmc_device_id_adjusted));

	sample_clock_mode_strings.push_back ("AutoSync");
	sample_clock_mode_strings.push_back ("Word Clock In");
	sample_clock_mode_strings.push_back ("Clock Master");

	sample_clock_mode_button = new GlobalClickBox 
		("SAMPLE CLOCK", sample_clock_mode_strings);

	sample_clock_mode_button->adjustment.value_changed.connect
		(slot(this,&ARDOUR_UI::sample_clock_mode_adjusted));

#if 0
	sample_format_strings.push_back ("16 bit");
	sample_format_strings.push_back ("20 bit");
	sample_format_strings.push_back ("24 bit");
	sample_format_strings.push_back ("32 bit");
	sample_format_strings.push_back ("32 bit FP");
	sample_format_strings.push_back ("64 bit FP");

	sample_format_button = new GlobalClickBox ("SAMPLE FORMAT",
						   sample_format_strings);
	
	rate_strings.push_back ("44.1kHz");
	rate_strings.push_back ("48kHz");
	rate_strings.push_back ("88.2kHz");
	rate_strings.push_back ("96kHz");

	rate_button = new GlobalClickBox ("SAMPLING RATE",
					  rate_strings);
#endif 

	adjuster_table.attach (xfade_vbox, 0, 1, 0, 1, 0, 0, 5, 5);
	adjuster_table.attach (preroll_vbox, 1, 2, 0, 1, 0, 0, 5, 5);
	adjuster_table.attach (postroll_vbox, 2, 3, 0, 1, 0, 0, 5, 5);

	adjuster_table.attach (*online_control_button, 0, 2, 1, 2, 
			     GTK_FILL|GTK_EXPAND, 0, 5, 5);
	adjuster_table.attach (*mmc_id_button, 2, 3, 1, 2, 
			     0, 0, 5, 5);
	adjuster_table.attach (*sample_clock_mode_button, 0, 2, 2, 3, 
			     GTK_FILL|GTK_EXPAND, 0, 5, 5);

#if 0
	adjuster_table.attach (*sample_format_button, 0, 2, 3, 4, 
			     GTK_FILL|GTK_EXPAND, 0, 5, 5);
	adjuster_table.attach (*rate_button, 0, 2, 4, 5, 
			     GTK_FILL|GTK_EXPAND, 0, 5, 5);
#endif

}

void
ARDOUR_UI::setup_transport ()

{
	nstring pixmap_path;

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->reverse_button_image.c_str());
	rewind_pixmap = new Gtk::Pixmap (pixmap_path);
	rewind_button.add (*rewind_pixmap);
	rewind_led = new LED ();
	rewind_led->black ();
	rewind_label.set_text ("REWIND");
	rewind_label.set_name ("TransportLabel");
	rewind_hbox.pack_start (*rewind_led, true, false, 10);
	rewind_vbox.pack_start (rewind_hbox, false, false, 5);
	rewind_vbox.pack_start (rewind_label, false, false);
	rewind_vbox.pack_start (rewind_button, false, false, 10);

	rewind_button.button_press_event.connect
		(slot(this,&ARDOUR_UI::transport_rewind));
	rewind_button.button_release_event.connect
		(slot(this,&ARDOUR_UI::transport_stop));

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->forward_button_image.c_str());
	forward_pixmap = new Gtk::Pixmap (pixmap_path);
	forward_button.add (*forward_pixmap);
	forward_led = new LED ();
	forward_led->black ();
	forward_label.set_text ("FAST FWD");
	forward_label.set_name ("TransportLabel");
	forward_hbox.pack_start (*forward_led, true, false);
	forward_vbox.pack_start (forward_hbox, false, false, 5);
	forward_vbox.pack_start (forward_label, false, false);
	forward_vbox.pack_start (forward_button, false, false, 10);

	forward_button.button_press_event.connect
		(slot(this,&ARDOUR_UI::transport_forward));
	forward_button.button_release_event.connect
		(slot(this,&ARDOUR_UI::transport_stop));

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->tape_button_image.c_str());
	tape_pixmap = new Gtk::Pixmap (pixmap_path);
	tape_button.add (*tape_pixmap);
	tape_label.set_text ("TAPE");
	tape_label.set_name ("TransportLabel");
	tape_vbox.pack_start (tape_hbox, false, false, 5);
	tape_vbox.pack_start (tape_label, false, false);
	tape_vbox.pack_start (tape_button, false, false, 10);

	tape_button.button_release_event.connect (slot (this,&ARDOUR_UI::tape_menu));

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->stop_button_image.c_str());
	stop_pixmap = new Gtk::Pixmap (pixmap_path);
	stop_button.add (*stop_pixmap);
	stop_led = new LED ();
	stop_led->green ();
	stop_label.set_text ("STOP");
	stop_label.set_name ("TransportLabel");
	stop_hbox.pack_start (*stop_led, true, false);
	stop_vbox.pack_start (stop_hbox, false, false, 5);
	stop_vbox.pack_start (stop_label, false, false);
	stop_vbox.pack_start (stop_button, false, false, 10);

	stop_button.button_release_event.connect
		(slot(this,&ARDOUR_UI::transport_stop));

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->play_button_image.c_str());
	roll_pixmap = new Gtk::Pixmap (pixmap_path);
	roll_button.add (*roll_pixmap);
	roll_led = new LED ();
	roll_led->black ();
	roll_label.set_text ("PLAY");
	roll_label.set_name ("TransportLabel");
	roll_hbox.pack_start (*roll_led, true, false);
	roll_vbox.pack_start (roll_hbox, false, false, 5);
	roll_vbox.pack_start (roll_label, false, false);
	roll_vbox.pack_start (roll_button, false, false, 10);

	roll_button.button_release_event.connect
		(slot(this,&ARDOUR_UI::transport_roll));

	pixmap_path = gtk_rc_find_pixmap_in_path (0, Config->record_button_image.c_str());
	rec_pixmap = new Gtk::Pixmap (pixmap_path);
	rec_button.add (*rec_pixmap);
	rec_led = new LED ();
	rec_led->black ();
	rec_label.set_text ("RECORD");
	rec_label.set_name ("TransportLabel");
	rec_hbox.pack_start (*rec_led, true, false);
	rec_vbox.pack_start (rec_hbox, false, false, 5);
	rec_vbox.pack_start (rec_label, false, false);
	rec_vbox.pack_start (rec_button, false, false, 10);

	rec_button.button_release_event.connect
		(slot(this,&ARDOUR_UI::transport_record));

	transport_packer.set_spacing (2);
	transport_packer.pack_start (tape_vbox, false, false, 5);
	transport_packer.pack_start (rewind_vbox, false, false, 5);
	transport_packer.pack_start (forward_vbox, false, false, 5);
	transport_packer.pack_start (stop_vbox, false, false, 5);
	transport_packer.pack_start (roll_vbox, false, false, 5);
	transport_packer.pack_end (rec_vbox, false, true, 20);

	transport_base.put (transport_packer, 0, 0);
	transport_base.set_name ("TransportBase");

	transport_frame.set_shadow_type (GTK_SHADOW_IN);
	transport_frame.set_name ("BaseFrame");
	transport_frame.add (transport_base);
}

void
ARDOUR_UI::setup_tape_menu ()

{
	using namespace Menu_Helpers;

	tape_popup_menu = new Menu();
	MenuList& items = tape_popup_menu->items ();

	items.push_back (MenuElem ("New Tape"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::new_tape));

	items.push_back (MenuElem ("Load"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::open_tape));

	items.push_back (MenuElem ("Unload"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::unload_tape));

	items.push_back (SeparatorElem());

	items.push_back (MenuElem ("Add track"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::tape_add_track));

	items.push_back (MenuElem ("Change length"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::tape_extend));

	items.push_back (MenuElem ("Erase Tape"));
	items.back()->activate.connect
		(slot (this, &ARDOUR_UI::wipe_tape));

	new_tape_dialog.set_position (GTK_WIN_POS_MOUSE);
	new_tape_dialog.set_title ("ARDOUR: New Tape Setup");
	new_tape_dialog.set_name ("NewTapeWindow");

	new_tape_name_label.set_name ("NewTapeLabel");
	new_tape_length_label.set_name ("NewTapeLabel");
	new_tape_tracks_label.set_name ("NewTapeLabel");

	new_tape_tracks.set_name ("NewTapeDisplay");
	new_tape_name.set_name ("NewTapeDisplay");
	new_tape_length.set_name ("NewTapeDisplay");

	new_tape_name_hbox.pack_start (new_tape_name_label, false,
				       false, 10);
	new_tape_name_hbox.pack_start (new_tape_name, true, true, 10);

	new_tape_length_hbox.pack_start (new_tape_length_label, false,
					 false, 10);
	new_tape_length_hbox.pack_start (new_tape_length, true, true, 10);

	new_tape_tracks_hbox.pack_start (new_tape_tracks_label, false,
					 false, 10);
	new_tape_tracks_hbox.pack_start (new_tape_tracks, true, true, 10);

	new_tape_button_hbox.pack_start (new_tape_ok, true, false);
	new_tape_button_hbox.pack_end (new_tape_cancel, true, false);

	new_tape_vpacker.set_spacing (10);
	new_tape_vpacker.pack_start (new_tape_name_hbox, false, false, 5);
	new_tape_vpacker.pack_start (new_tape_length_hbox, false, false);
	new_tape_vpacker.pack_start (new_tape_tracks_hbox, false, false);
	new_tape_vpacker.pack_start (new_tape_button_hbox, false, false, 5);

	new_tape_dialog.add (new_tape_vpacker);

	new_tape_dialog.delete_event.connect (slot (this, &ARDOUR_UI::new_tape_window_deleted));
	new_tape_cancel.clicked.connect(slot(this,&ARDOUR_UI::cancel_new_tape));
	new_tape_ok.clicked.connect(slot(this,&ARDOUR_UI::accept_new_tape));
}

void
ARDOUR_UI::setup_clock ()

{
	clock_ui = new AudioClock_UI (mcd);
	clock_packer.pack_start (*clock_ui, false, false, 10);
	clock_base.put (clock_packer, 0, 0);
	clock_base.set_name ("ClockBase");
	clock_frame.set_shadow_type (GTK_SHADOW_IN);
	clock_frame.set_name ("BaseFrame");
	clock_frame.add (clock_base);

}

int
ARDOUR_UI::setup_wipe_window ()

{
	wipe_commit_button = new Gtkmmext::PixmapButton
		(push_button_pix(), "GlobalButton");
	wipe_cancel_button = new Gtkmmext::PixmapButton
		(push_button_pix(), "GlobalButton");

	wipe_commit_label.set_name ("WipeLabel");
	wipe_cancel_label.set_name ("WipeLabel");

	wipe_cancel_label.set_text ("Cancel");

	wipe_cancel_box.pack_start (*wipe_cancel_button, false, false, 20);
	wipe_cancel_box.pack_start (wipe_cancel_label, false, false, 20);

	wipe_commit_box.pack_start (*wipe_commit_button, false, false, 20);
	wipe_commit_box.pack_start (wipe_commit_label, false, false, 20);

	wipe_box.pack_start (wipe_commit_box, false, false, 20);
	wipe_box.pack_start (wipe_cancel_box, false, false, 20);

	wipe_frame.set_shadow_type (GTK_SHADOW_OUT);
	wipe_frame.set_name ("BaseFrame");
	wipe_frame.add (wipe_box);

	wipe_window.add (wipe_frame);
	wipe_window.set_name ("WipeWindow");
	wipe_window.set_position (GTK_WIN_POS_MOUSE);
	wipe_window.set_modal (true);

	wipe_commit_button->clicked.connect (slot (this, &ARDOUR_UI::wipe_tape));
	wipe_cancel_button->clicked.connect (slot (this, &ARDOUR_UI::wipe_cancel));

	wipe_window.delete_event.connect (slot (this, &ARDOUR_UI::wipe_delete));
	
	return 0;
}

void
ARDOUR_UI::build_menu_bar ()

{
	using namespace Gtk;
	using namespace Gtk::Menu_Helpers;

	menu_bar.set_name ("RecorderMenuBar");

	MenuList& items = menu_bar.items();

	/* file menu */
	
	Menu *file_menu = new Menu();
	MenuList& file_items = file_menu->items();
	file_items.push_back (MenuElem ("Save", slot (this, &ARDOUR_UI::save_state)));
	file_items.push_back (MenuElem ("Save as"));
	file_items.push_back (MenuElem ("Export"));
	file_items.push_back (SeparatorElem());
	file_items.push_back (MenuElem ("Quit", slot (this, &ARDOUR_UI::quit)));
	items.push_back (MenuElem ("File", *file_menu));

	/* windows menu */

	Menu *window_menu = new Menu();
	MenuList& window_items = window_menu->items();
	window_items.push_back (MenuElem ("Editor"));
	window_items.back()->activate.connect
		(bind (slot (this, &ARDOUR_UI::switch_to_notebook_page), 1));

	window_items.push_back (SeparatorElem());

	window_items.push_back 
		(CheckMenuElem 
		 ("Locations",
		  slot (this, &ARDOUR_UI::toggle_location_window)));

	window_items.push_back 
		(CheckMenuElem 
		 ("Controls",
		  slot (this, &ARDOUR_UI::toggle_control_window)));

	items.push_back (MenuElem ("Windows", *window_menu));
}

