/* gtkedlwavebuffer.c: class that stores data in memory
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>

#include <glib.h>
#include <gtk/gtk.h>
#include "gtkedlwavebuffer.h"


static void gtk_edl_wave_buffer_class_init (GtkEDLWaveBufferClass *klass);
static void gtk_edl_wave_buffer_init       (GtkEDLWaveBuffer      *memory);
static gboolean gtk_edl_wave_buffer_is_editable     (GtkWaveBuffer *wavebuffer);
static guint32 gtk_edl_wave_buffer_get_rate         (GtkWaveBuffer *wavebuffer);
static GtkWaveBufferType gtk_edl_wave_buffer_get_datatype  (GtkWaveBuffer *wavebuffer);
static guint32 gtk_edl_wave_buffer_get_length       (GtkWaveBuffer *wavebuffer);
static guint32 gtk_edl_wave_buffer_get_num_channels (GtkWaveBuffer *wavebuffer);
static void gtk_edl_wave_buffer_get_samples         (GtkWaveBuffer *wavebuffer,
                                                        guint32 start,
                                                        guint32 length,
                                                        guint32 channel_mask,
                                                        gpointer data);

static gint gtk_edl_wave_buffer_insert              (GtkEditableWaveBuffer *editable,
                                                        guint32 start,
                                                        piece_t *piece);
static gint gtk_edl_wave_buffer_insert_sequence     (GtkEditableWaveBuffer *editable,
                                                        guint32 start,
                                                        sequence_t *seq);
static gint gtk_edl_wave_buffer_replace             (GtkEditableWaveBuffer *editable,
                                                        guint32 start,
                                                        piece_t *piece);
static gint gtk_edl_wave_buffer_replace_sequence    (GtkEditableWaveBuffer *editable,
                                                        guint32 start,
						        sequence_t *sequence);
static gint gtk_edl_wave_buffer_delete              (GtkEditableWaveBuffer *editable,
                                                        guint32 start,
                                                        guint32 length);

static gint gtk_edl_wave_buffer_undo                (GtkEditableWaveBuffer *editable,
                                                        guint32 undo_depth);
static gint gtk_edl_wave_buffer_redo                (GtkEditableWaveBuffer *editable,
                                                        guint32 redo_depth);

GtkType
gtk_edl_wave_buffer_get_type (void)
{
  static GtkType memory_type = 0;

  if (!memory_type)
    {
      static const GtkTypeInfo memory_info =
      {
        "GtkEDLWaveBuffer",
        sizeof (GtkEDLWaveBuffer),
        sizeof (GtkEDLWaveBufferClass),
        (GtkClassInitFunc) gtk_edl_wave_buffer_class_init,
        (GtkObjectInitFunc) gtk_edl_wave_buffer_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL
      };

      memory_type = gtk_type_unique (GTK_TYPE_EDITABLE_WAVE_BUFFER, &memory_info);
    }

  return memory_type;
}


static void
gtk_edl_wave_buffer_finalize (GtkObject *obj)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (obj);

  g_print ("Finalize called.\n");
}


static void
gtk_edl_wave_buffer_class_init (GtkEDLWaveBufferClass *klass)
{
  GtkObjectClass *object_class;
  GtkWaveBufferClass *wavebuffer_class;
  GtkEditableWaveBufferClass *editable_class;

  object_class = (GtkObjectClass *) klass;
  wavebuffer_class = (GtkWaveBufferClass *) klass;
  editable_class = (GtkEditableWaveBufferClass *) klass;

  object_class->finalize = gtk_edl_wave_buffer_finalize;

  wavebuffer_class->get_rate = gtk_edl_wave_buffer_get_rate;
  wavebuffer_class->get_datatype = gtk_edl_wave_buffer_get_datatype;
  wavebuffer_class->get_length = gtk_edl_wave_buffer_get_length;
  wavebuffer_class->get_num_channels = gtk_edl_wave_buffer_get_num_channels;
  wavebuffer_class->get_samples = gtk_edl_wave_buffer_get_samples;
  editable_class->insert = gtk_edl_wave_buffer_insert;
  editable_class->insert_sequence = gtk_edl_wave_buffer_insert_sequence;
  editable_class->replace = gtk_edl_wave_buffer_replace;
  editable_class->replace_sequence = gtk_edl_wave_buffer_replace_sequence;
  editable_class->remove = gtk_edl_wave_buffer_delete;
  editable_class->undo = gtk_edl_wave_buffer_undo;
  editable_class->redo = gtk_edl_wave_buffer_redo;
}


static void
gtk_edl_wave_buffer_init (GtkEDLWaveBuffer *edlbuf)
{
  edlbuf->n_channels = 0;
  edlbuf->d_type = GTK_WAVE_BUFFER_TYPE_NULL;
  edlbuf->rate = 0;
}


static guint32
gtk_edl_wave_buffer_get_rate (GtkWaveBuffer *wavebuffer)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (wavebuffer);

  return edlbuf->rate;
}


static GtkWaveBufferType
gtk_edl_wave_buffer_get_datatype (GtkWaveBuffer *wavebuffer)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (wavebuffer);

  return edlbuf->d_type;
}


static guint32
gtk_edl_wave_buffer_get_length (GtkWaveBuffer *wavebuffer)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (wavebuffer);

  return edlbuf->edl->length / gtk_wave_buffer_type_width (edlbuf->d_type);
}


static guint32
gtk_edl_wave_buffer_get_num_channels (GtkWaveBuffer *wavebuffer)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (wavebuffer);

  return edlbuf->n_channels;
}


static void
gtk_edl_wave_buffer_get_samples (GtkWaveBuffer *wavebuffer,
                                    guint32  start,
                                    guint32  length,
                                    guint32  channel_mask,
                                    gpointer data)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (wavebuffer);
  guint32 i, offset;

  /* XXX NON-INTERLEAVED DATA */

  offset = 0;

  for (i = 0; i < edlbuf->n_channels; i++) {
	  if (channel_mask & (1<<i)) {
		  sequence_read (edlbuf->edl, data+offset, start, length);
		  offset += length;
	  }
  }
}

static gint
gtk_edl_wave_buffer_insert (GtkEditableWaveBuffer *editable,
			    guint32 start,
			    piece_t *piece)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);
  GRange range;

  sequence_insert_piece (edlbuf->edl, start, piece);
  g_range_set (&range, start, piece->length * 2);
  gtk_signal_emit (GTK_OBJECT (editable), modified_signal, &range);
  gtk_editable_wave_buffer_set_modified (editable, start, piece->length * 2);
  return 0;
}

static gint
gtk_edl_wave_buffer_insert_sequence (GtkEditableWaveBuffer *editable,
				     guint32 start,
				     sequence_t *seq)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);
  GRange range;
  
  sequence_insert_sequence (edlbuf->edl, start, seq);
  g_range_set (&range, start, seq->length * 2);
  gtk_signal_emit (GTK_OBJECT (editable), modified_signal, &range);
  gtk_editable_wave_buffer_set_modified (editable, start, seq->length * 2);
  return 0;
}

static gint
gtk_edl_wave_buffer_replace (GtkEditableWaveBuffer *editable,
			     guint32 start,
			     piece_t *piece)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);
  GRange range;
  
  sequence_replace_piece (edlbuf->edl, start, piece);
  g_range_set (&range, start, piece->length);
  gtk_signal_emit (GTK_OBJECT (editable), modified_signal, &range);
  gtk_editable_wave_buffer_set_modified (editable, start, piece->length);
  return 0;
}

static gint
gtk_edl_wave_buffer_replace_sequence (GtkEditableWaveBuffer *editable,
				      guint32 start,
				      sequence_t *seq)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);
  GRange range;
  
  sequence_replace_sequence (edlbuf->edl, start, seq);
  
  g_range_set (&range, start, seq->length);
  gtk_signal_emit (GTK_OBJECT (editable), modified_signal, &range);
  gtk_editable_wave_buffer_set_modified (editable, start, seq->length);

  return 0;
}

static gint
gtk_edl_wave_buffer_redo (GtkEditableWaveBuffer *editable,
			  guint32 redo_depth)
{
	GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);

	while (redo_depth--) {
		if (sequence_redo (edlbuf->edl)) {
			return -1;
		}
	}

	gtk_editable_wave_buffer_set_modified (editable, 0, edlbuf->edl->length);
	return 0;
}

static gint
gtk_edl_wave_buffer_undo (GtkEditableWaveBuffer *editable,
			  guint32 undo_depth)
{
	GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);

	while (undo_depth--) {
		if (sequence_undo (edlbuf->edl)) {
			return -1;
		}
	}

	gtk_editable_wave_buffer_set_modified (editable, 0, edlbuf->edl->length);
	return 0;
}

static gint
gtk_edl_wave_buffer_delete (GtkEditableWaveBuffer *editable,
			    guint32 start,
			    guint32 length)
{
  GtkEDLWaveBuffer *edlbuf = GTK_EDL_WAVE_BUFFER (editable);
  guint32 mod_length;
  GRange range;
  
  mod_length = edlbuf->edl->length - start;

  sequence_delete (edlbuf->edl, start, length);
  g_range_set (&range, start, mod_length);
  gtk_signal_emit (GTK_OBJECT (editable), delete_data_signal, &range);
  gtk_editable_wave_buffer_set_modified (editable, start, mod_length);

  return 0;
}

GtkObject *
gtk_edl_wave_buffer_new (sequence_t *edl,
			 guint16 n_channels,
			 GtkWaveBufferType d_type,
			 guint32 rate)
{
  GtkEDLWaveBuffer *wavebuffer;

  wavebuffer = gtk_type_new (GTK_TYPE_EDL_WAVE_BUFFER);
  wavebuffer->n_channels = n_channels;
  wavebuffer->d_type = d_type;
  wavebuffer->rate = rate;
  wavebuffer->edl = edl;
  
  return GTK_OBJECT (wavebuffer);
}
