/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: location.cc,v 1.9 2000/10/13 18:37:23 pbd Exp $
*/

#include <stdio.h> /* for sprintf */
#include <unistd.h>
#include <errno.h>
#include <time.h>

#include <fstream>

#include "location.h"
#include "tape.h"

#include <pbd/stl_delete.h>

static ostream &operator<< (ostream&, Location&);

LocationStack::LocationStack ()

{
	current_location_index = 0;
	pthread_mutex_init (&lock, 0);
	
	add ("last record start");
	add ("last playback start");
}

LocationStack::~LocationStack ()

{
	vector_delete (&locations);
}

void
LocationStack::add (const string &name)

{
	Location *l;

	pthread_mutex_lock (&lock);
	l = new Location (0, 0, name);
	locations.push_back (l);
	current_location_index = locations.size() - 1;

	pthread_mutex_unlock (&lock);
	
	changed (locations[current_location_index]);
}

void
LocationStack::add ()

{
	Location *l;
	char buf[64];

	pthread_mutex_lock (&lock);
	sprintf (buf, "unnamed location %d", locations.size() + 1);
	l = new Location (0, 0, buf);
	locations.push_back (l);
	current_location_index = locations.size() - 1;

	pthread_mutex_unlock (&lock);
	
	changed (locations[current_location_index]);
}

void
LocationStack::up ()

{
	pthread_mutex_lock (&lock);
	if (current_location_index < locations.size() - 1) {
		current_location_index++;
		pthread_mutex_unlock (&lock);
		changed (locations[current_location_index]);
		return;
	} 
	pthread_mutex_unlock (&lock);
	add ();
}

void
LocationStack::down ()

{
	pthread_mutex_lock (&lock);
	if (current_location_index > 0) {
		current_location_index--;
		pthread_mutex_unlock (&lock);
		changed (locations[current_location_index]);
		return;
	}
	pthread_mutex_unlock (&lock);
}

int
LocationStack::save (ARDOUR::Tape &tape)

{
	if (locations.size() == 0) {
		return 0;
	}

	pthread_mutex_lock (&lock);

	string path = tape.path();
	path += "/locations";
	
	ofstream out;

	out.open (path.c_str(), ios::out|ios::trunc);

	if (!out) {
		error << "Locations: could not open "
		      << path
		      << " to store locations\n"
		      << endmsg;
		pthread_mutex_unlock (&lock);
		return -1;
	}

	vector<Location *>::iterator iter;

	time_t now;
	time (&now);

	char hostname[64];
	gethostname (hostname, sizeof (hostname) - 1);

	out << "# Locations saved on " << hostname << " at " << ctime (&now);

	for (iter  = locations.begin();
	     iter != locations.end();
	     iter++) {
		out << *(*iter);
		out << '\n';
	}

	pthread_mutex_unlock (&lock);
	return 0;
}	

int
LocationStack::restore (ARDOUR::Tape &tape)

{
	string path = tape.path();
	path += "/locations";

	if (access (path.c_str(), R_OK)) {
		if (errno == ENOENT) {
			return 0;  /* XXX is this OK ? */
		} else {
			error << "Locations: cannot read locations file: "
			      << strerror (errno)
			      << endmsg;
			return -1;
		}
	}

	ifstream in (path.c_str());

	if (!in) {
		error << "Locations: could not open "
		      << path
		      << " to retrieve locations\n"
		      << endl;
		return -1;
	}

	pthread_mutex_lock (&lock);
	vector_delete (&locations);
	tape.delete_auto_locations ();
	
	while (in) {
		Location *loc;
		char line[512];
		char name[512];
		size_t start;
		size_t end;
		int punch;
		int loop;

		in.getline (line, sizeof (line), '\n');

		if (in.gcount() == 0) {
			break;
		}
		
		if (line[0] == '\0' || line[0] == '#') {
			continue;
		}
		
		if (sscanf (line, "\"%[^\"]\" %u %u %d %d", 
			    name, &start, &end, &punch, &loop) != 5) {
			error << "Locations: format error in file: ["
			      << line
			      << ']'
			      << endmsg;
			break;
		}

		loc = new Location;

		loc->set_name (name);
		loc->set_start (start);
		loc->set_end (end);

		locations.push_back (loc);

		if (punch) {
			tape.set_auto_punch_location (loc);
		} else {
			loc->set_auto_punch (false);
		}

		if (loop) {
			tape.set_auto_loop_location (loc);
		} else {
			loc->set_auto_loop (false);
		}
	}

	if (locations.size() == 0) {
		fatal << "Gack! programmer error ... talk to pbd@op.net"
		      << endmsg;
		/*NOTREACHED*/
	}

	current_location_index = 0;
	pthread_mutex_unlock (&lock);

	changed (locations[current_location_index]);

	return 0;
}	

static ostream &
operator<< (ostream& o, Location& l)
{
	o << '"' << l.name() << '"'
	  << ' '
	  << l.start()
	  << ' '
	  << l.end()
	  << ' '
	  << l.is_auto_punch()
	  << ' '
	  << l.is_auto_loop ();

	return o;
}
