/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: location.h,v 1.8 2000/10/13 18:37:23 pbd Exp $
*/

#ifndef __ardour_location_h__
#define __ardour_location_h__

#include <string>
#include <vector>
#include <iostream>

#include <sys/types.h>

#include <sigc++/signal_system.h>
#include <pbd/pthread_spinlock.h>

namespace ARDOUR {
	class Tape;
}

class Location : public SigC::Object
{
  public:
	Location (size_t sample_start,
		  size_t sample_end,
		  const string &name)
		: _name (name),
		  _start (sample_start),
		  _end (sample_end) { 
		_is_auto_punch = false;
		_is_auto_loop = false;
	}

	Location () {
		_start = 0;
		_end = 0;
		_is_auto_punch = false;
		_is_auto_loop = false;
	}

	size_t start() { return _start; }
	size_t end() { return _end; }
	
	void set_start (size_t s) { _start = s; start_changed(this); }
	void set_end (size_t s) { _end = s; end_changed(this); }

	const string &name() { return _name; }
	void set_name (const string &str) { _name = str; name_changed(this); }

	void set_auto_punch (bool yn)  { _is_auto_punch = yn; }
	void set_auto_loop (bool yn) { _is_auto_loop = yn; }

	bool is_auto_punch ()  { return _is_auto_punch; }
	bool is_auto_loop () { return _is_auto_loop; }

	SigC::Signal1<void,Location*> name_changed;
	SigC::Signal1<void,Location*> end_changed;
	SigC::Signal1<void,Location*> start_changed;

  private:
	string _name;
	size_t _start;
	size_t _end;
	bool _is_auto_punch;
	bool _is_auto_loop;
};

class LocationStack : public SigC::Object

{
  public:
	LocationStack ();
	~LocationStack ();

	void add (const string &name);
	void add ();

	void up ();
	void down ();

	int save (ARDOUR::Tape &);
	int restore (ARDOUR::Tape &);

	Location *current () { 
		Location *l;
		pthread_mutex_spinlock (&lock);
		l = locations[current_location_index];
		pthread_mutex_unlock (&lock);
		return l;
	}

	Location *last_roll_location() {
		Location *l;
		pthread_mutex_spinlock (&lock);
		l = locations[1];
		pthread_mutex_unlock (&lock);
		return l;
	}

	Location *last_record_location() {
		Location *l;
		pthread_mutex_spinlock (&lock);
		l = locations[0];
		pthread_mutex_unlock (&lock);
		return l;
	}

	SigC::Signal1<void,Location *> changed;

  private:
	vector<Location *>      locations;
	size_t                  current_location_index;
	pthread_mutex_t         lock;
};

#endif /* __ardour_location_h__ */
