/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: seq.c,v 1.9 2000/11/04 16:35:47 pbd Exp $
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "seq.h"
#include "filebuffer.h"
#include "constbuffer.h"

/* This is a set of "objects"/"functions" designed to implement
   extremely efficient editing of arbitarily sized data streams.
   
   There are 3 layers to the model used here:
   
   buffer_t: an abstract representation of some data stream.

              Precisely what the items in the stream are, where they
              come from, how they get here: this is all completely
              irrelevant to the API. buffer_t's are created using
	      a specific kind of buffer creation function. the
	      most common is file_buffer_new(); see filebuffer.h

   piece_t: describes a contiguous region within a buffer_t.

   sequence_t: an ordered list of piece_t's that, taken together,
               describes the current state of the data stream.

   There are operations to insert in, delete and replace any part
   of the data stream. AT NO TIME is the underlying data ever
   modified in any way. Editing operations simply alter the
   list of piece_t's making up a sequence. In fact, even the very
   existence of ANY underlying data only matters when attempting to
   read the data stream via sequence_read() or writing via
   sequence_write().

   A sequence_t also has infinite undo and redo capabilities. Every
   time an editing (insert, delete, replace) operation is performed,
   the state of the sequence_t is saved. The operation may be undone
   with sequence_undo(); each time an operation is undone, it is also
   put on the redo list, and may be redone with sequence_redo.  
*/

static buffer_t *silent_buffer = 0;

buffer_t *
buffer_new (BufferType (*type)(buffer_t *),
	    gchar *(*id)(buffer_t *),
	    guint32 (*len)(), 
	    guint32 (*read)(buffer_t*, buffer_data_t *, guint32, guint32),
	    buffer_t *(*clone)(buffer_t *),
	    guint32 (*append)(buffer_t*, buffer_t*, guint32, guint32),
	    piece_t *(*append_data)(buffer_t*, buffer_data_t*, guint32),
	    void (*close)(buffer_t *),
	    gpointer data)
{
	buffer_t *buf = (buffer_t *) g_malloc (sizeof (buffer_t));
	
	buf->type = type;
	buf->id = id;
	buf->length = len;
	buf->read = read;
	buf->clone = clone;
	buf->append = append;
	buf->append_data = append_data;
	buf->close = close;

	buf->private_data = data;
	buf->refcnt = 1;

	return buf;
}

static gboolean
buffer_equal (buffer_t *buf, BufferType type, const gchar *path)

{
	gchar *id;
	gboolean ret = FALSE;

	if (buf->type (buf) == type) {
		id = buf->id (buf);
		ret = (strcmp (id, path) == 0);
		g_free (id);
	}
	return ret;
}

static buffer_t *
buffer_ref (buffer_t *buf)
{
	buf->refcnt++;
	return buf;
}

static void
buffer_free (buffer_t *buf)
{
	if (buf->refcnt && --buf->refcnt == 0) {
		buf->close (buf);
		g_free (buf);
	}
}
	

/***********************************************************************

     PIECE

     A piece_t holds a pointer to some kind of data, together with
     a start offset into that data, and a count of how many data
     items are part the contiguous region that the `piece' describes.

  ***********************************************************************/

piece_t *
piece_new (buffer_t *b, guint32 s, guint32 l) 
{
	piece_t *p = (piece_t *) g_malloc (sizeof (piece_t));

	p->buffer = buffer_ref (b);
	p->start = s;
	p->length = l;
	p->refcnt = 1;

	return p;
}

void
piece_free (piece_t *piece)

{
	if (piece->refcnt && --piece->refcnt == 0) {
		buffer_free (piece->buffer);
		g_free (piece);
	}
}

void
piece_ref (piece_t *p)

{
	p->refcnt++;
}

static piece_t *
piece_dup (piece_t *old)

{
	piece_t *p = (piece_t *) g_malloc (sizeof (piece_t));

	p->buffer = buffer_ref (old->buffer);
	p->start = old->start;
	p->length = old->length;
	p->refcnt = 1;

	return p;
}

static GList *
piece_list_copy (GList *oldlist)

{
	GList *s;
	GList *newlist = NULL;
	piece_t *newpiece;

	for (s = oldlist; s; s = g_list_next (s)) {
		newpiece = piece_dup ((piece_t *) s->data);
		newlist = g_list_append (newlist, newpiece);
	}

	return newlist;
}

/***********************************************************************
 
       SEQUENCE

       A sequence is an ordered list of pieces, together with
       undo and redo states, and an overall length.
  
 **********************************************************************/

static sequence_state_t *
sequence_state_new (GList *pieces, guint32 length, float gain, sequence_op_t *op)

{
	sequence_state_t *undo = (sequence_state_t *) g_malloc (sizeof (sequence_state_t));
	GList *listp;
			
	undo->pieces = piece_list_copy (pieces);
	undo->length = length;
	undo->gain = gain;
	undo->op = op;

	return undo;
}

static void
sequence_state_free (sequence_state_t *state)

{
	/* Do ***NOT*** free the list. The list itself is either in a
	   state stack and/or in use by a sequence at this time as its
	   own list.
	*/

	if (state->op->piece) {
		piece_free (state->op->piece);
	}
	g_free (state->op);
	g_free (state);
}

static gchar *
sequence_op_print_short (sequence_op_t *op, gchar *buf)

{
	gchar *id = NULL;

	switch (op->type) {
	case SequenceInsert:
		sprintf (buf, "I %c \"%s\" %u %u %u",
			 op->piece->buffer->type (op->piece->buffer),
			 (id = op->piece->buffer->id (op->piece->buffer)), 
			 op->piece->start,
			 op->cnt, op->start);
		break;

	case SequenceReplace:
		sprintf (buf, "R %c \"%s\" %u %u %u",
			 op->piece->buffer->type (op->piece->buffer),
			 (id = op->piece->buffer->id (op->piece->buffer)),
			 op->piece->start,
			 op->cnt, op->start);
		break;

	case SequenceDelete:
		sprintf (buf, "D %u %u", op->cnt, op->start);
		break;

	case SequenceSetGain:
		sprintf (buf, "G %f\n", op->gain);
		break;
	}

	if (id) {
		g_free (id);
	}

	return buf;
}


static sequence_op_t *
sequence_op_new (SequenceOpType t, guint32 start, guint32 cnt, piece_t *piece, float gain) 
{
	sequence_op_t *op = (sequence_op_t *) g_malloc (sizeof (sequence_op_t));

	op->type = t;
	op->start = start;

	switch (t) {
	case SequenceDelete:
		op->cnt = cnt;
		op->piece = NULL;
		break;
	case SequenceSetGain:
		op->cnt = cnt;
		op->piece = NULL;
		op->gain = gain;
		break;
	default:
		op->piece = piece;
		piece_ref (piece);
		op->cnt = piece->length;
	}
	return op;
}

static sequence_op_t *
sequence_op_dup (sequence_op_t *op)

{
	return sequence_op_new (op->type, op->start, op->cnt, op->piece, op->gain);
}

static buffer_t *
get_buffer (GList **list, BufferType buftype, const gchar *path)

{
	buffer_t *buf = NULL;
	GList *listp;

	for (listp = *list; listp; listp = g_list_next (listp)) {
		buf = (buffer_t *) listp->data;
		if (buffer_equal (buf, buftype, path)) {
			return buf;
		}
	}
	
	switch (buftype) {
	case FileBuffer:
		buf = file_buffer_new (path);
		break;
	case ConstBuffer:
		buf = const_buffer_new (atoi (path));
		break;
	}
	
	if (buf) {
		*list = g_list_append (*list, buf);
	}

	return buf;
}

static void
sequence_dump (sequence_t *seq)

{
	GList *listp;
	piece_t *p;
	gchar *id;
	guint32 n; 

	printf ("\n\n- START -------------------------------\n");
	for (n = 0, listp = seq->pieces; listp; listp = g_list_next (listp), n++) {
		p = (piece_t *) listp->data;
		printf ("piece #%u 0x%x: buffer = %c \"%s\", start in buffer = %u, length = %u\n",
			n + 1, p,
			p->buffer->type (p->buffer),
			(id = p->buffer->id (p->buffer)),
			p->start,
			p->length);
		g_free (id);
	}
	printf ("- END ---------------------------------\n\n");
}
			
sequence_t *
sequence_new (const gchar *path, buffer_t *existing_buffer)
{
	sequence_t *seq;
	FILE *f;
	char cmd;
	char bufchar;
	char file[PATH_MAX+1];
	guint32 piece_start;
	guint32 piece_length;
	guint32 at;
	buffer_t *buf;
	piece_t *piece;
	GList *buflist = NULL;
	int x;
	char edlpath[PATH_MAX+1];
	float newgain;

	if (silent_buffer == 0) {
		silent_buffer = const_buffer_new (0);
	}

	if (path == NULL) {
		if (existing_buffer == NULL) {
			fprintf (stderr, "EDL: no path, no existing buffer!\n");
			return NULL;
		}
	} else {
		sprintf (edlpath, "%s.edl", path);

		if ((f = fopen (edlpath, "a+")) == NULL) {
			fprintf (stderr, "EDL: cannot open edl file \"%s\" (%s)\n", 
				 edlpath, strerror (errno));
			return NULL;
		}
		fseek (f, 0, SEEK_SET);
	} 

	seq = (sequence_t *) g_malloc (sizeof (sequence_t));
	seq->pieces =  NULL;
	seq->state_ptr = NULL;
	seq->length = 0;
	seq->gain = 1;
	seq->loading = TRUE;
	seq->path = path ? g_strdup (edlpath) : NULL;
	seq->backup_path = (const gchar *) g_malloc (strlen (edlpath) + 6);
	strcpy ((gchar *) seq->backup_path, edlpath);
	strcat ((gchar *) seq->backup_path, ".save");

	if (path) {
		if ((seq->default_buffer = file_buffer_new (path)) == NULL) {
			goto out;
		}
	} else {
		seq->default_buffer = existing_buffer;
		goto done;
	}

	while (!feof (f)) {
		if (fscanf (f, " %c", &cmd) != 1) {
			break;
		}

		switch (cmd) {
		case 'I':
			if ((x = fscanf (f, " %c \"%[^\"]\" %u %u %u", 
					 &bufchar, file,
					 &piece_start, &piece_length, &at)) != 5) {
				fprintf (stderr, "bad cmd suffix %d\n", x);
				goto out;
			}
			if ((buf = get_buffer (&buflist, (BufferType) bufchar, file)) == NULL) {
				fprintf (stderr, "no buffer\n");
				goto out;
			}
			piece = piece_new (buf, piece_start, piece_length);
			sequence_insert_piece (seq, at, piece);
			break;

		case 'R':
			if (fscanf (f, " %c \"%[^\"]\" %u %u %u", 
				    &bufchar, file,
				    &piece_start, &piece_length, &at) != 5) {
				goto out;
			}
			if ((buf = get_buffer (&buflist, (BufferType) bufchar, file)) == NULL) {
				printf ("buffer not found\n");
				goto out;
			}
			piece = piece_new (buf, piece_start, piece_length);
			sequence_replace_piece (seq, at, piece);
			break;

		case 'D':
			if (fscanf (f, "%u %u\n", &piece_start, &piece_length) != 2) {
				goto out;
			}

			sequence_delete (seq, piece_start,
					 piece_length);
			break;

		case 'G':
			if (fscanf (f, " %f\n", &newgain) != 1) {
				goto out;
			}
			sequence_set_gain (seq, newgain);
			break;
					   
		default:
			fprintf (stderr, "unknown cmd %c\n", cmd);
			goto out;
		}
	}

	fclose (f);
	g_list_free (buflist);

//	sequence_dump (seq);

  done:
	seq->loading = FALSE;
	return seq;

  out:
	fclose (f);
	g_list_free (buflist);
	sequence_free (seq);
	return NULL;
}

void
sequence_free (sequence_t *seq)

{
	GList *listp;

	for (listp = seq->pieces; listp; listp = g_list_next (listp)) {
		piece_free ((piece_t *) listp->data);
	}

	g_list_free (seq->pieces);

	for (listp = g_list_first (seq->state_ptr); listp; listp = g_list_next (listp)) {
		sequence_state_free ((sequence_state_t *) listp->data);
	}
	g_list_free (g_list_first (seq->state_ptr));

	if (seq->path) {
		g_free ((gchar *) seq->path);
	}
	g_free (seq);
}

static int
sequence_store_as_edl (sequence_t *seq, const gchar *path)

{
	FILE *f;
	GList *state;
	char buf[1024];

	if ((f = fopen (path, "w")) == NULL) {
		return -1;
	}

	/* Print it in temporal order. Since its a stack, this means in reverse.
	*/

	for (state = g_list_last (seq->state_ptr); state ; state = g_list_previous (state)) {
		sequence_op_print_short (((sequence_state_t *) state->data)->op, buf);
		fprintf (f, "%s\n", buf);

		if (ferror (f)) {
			unlink (path);
			fclose (f);
			return -1;
		}

		if (state == seq->state_ptr) {
			break;
		}
	}

	fclose (f);
	return 0;
}

static void
sequence_merge_adjacent_pieces (sequence_t *seq)
{
	GList *node, *tmp;
	piece_t *piece;
	piece_t *prev_piece;

	/* This finds all pieces that are contiguous in the buffer and
	   the sequence, and merges them into a single piece.

	   We call it before we save the sequence state, so that the
	   state lists are of minimal length. Note that we do not
	   attempt to free the pieces liberated in this way, because
	   they may be referenced elsewhere (XXX BS - what are the
	   refcnts for anyway ?)
	*/

	for (prev_piece = 0, node = seq->pieces; node; ) {

		piece = (piece_t *) node->data;
		tmp = g_list_next (node);
		
		if (prev_piece && 
		    prev_piece->buffer == piece->buffer && 
		    prev_piece->start + prev_piece->length == piece->start) {

			prev_piece->length += piece->length;
			g_list_remove_link (seq->pieces, node);
			g_list_free_1 (node);

		} else {
			prev_piece = piece;
		}
		
		node = tmp;
	}
}

static void
sequence_save_state (sequence_t *seq, SequenceOpType type, guint32 start, guint32 cnt, piece_t *piece, float gain)

{
	sequence_op_t *op;
	sequence_state_t *state;	

	op = sequence_op_new (type, start, cnt, piece, gain);

	/* if the state ptr is not the top of the stack,
	   release everything above it.
	*/

	if (g_list_previous (seq->state_ptr)) {
		GList *listp;
		GList *tmp;

		for (listp = g_list_first (seq->state_ptr); 
		     listp != seq->state_ptr; ) {
			sequence_state_free ((sequence_state_t *)
					     listp->data);
			tmp = g_list_next (listp);
			g_list_remove_link (g_list_first (seq->state_ptr), listp);
			g_list_free_1 (listp);
			listp = tmp;
		}
	}

	state = sequence_state_new (seq->pieces, seq->length, seq->gain, op);
	seq->state_ptr = g_list_prepend (seq->state_ptr, state);

	/* Do NOT attempt to overwrite the EDL file when we're
	   in the middle of loading it. That's a disaster :)
	*/
	
	if (seq->path && !seq->loading) {
		if (sequence_store_as_edl (seq, seq->backup_path)) {
			fprintf (stderr, "could not save EDL state to %s.\n", seq->backup_path);
		}
	}
}

int
sequence_undo (sequence_t *seq)

{
	sequence_state_t *state;

	if (g_list_next (seq->state_ptr) == 0) {
		return 1;
	}
	
	state = (sequence_state_t *) seq->state_ptr->data;
	seq->state_ptr = g_list_next (seq->state_ptr);

	seq->pieces = state->pieces;
	seq->length = state->length;
	seq->gain = state->gain;

	return 0;
}

int
sequence_redo (sequence_t *seq)

{
	sequence_state_t *state;

	if (g_list_previous (seq->state_ptr) == 0) {
		return 1;
	}

	/* Move the current state pointer back up the stack */

	seq->state_ptr = g_list_previous (seq->state_ptr);
	state = (sequence_state_t *) seq->state_ptr->data;

	seq->pieces = state->pieces;
	seq->length = state->length;
	seq->gain = state->gain;

	return 0;
}

static void 
sequence_insert_piece_before (sequence_t *seq, GList *sublist, piece_t *p) {
	if (sublist == seq->pieces) {
		seq->pieces = g_list_prepend (seq->pieces, p);
	} else if (sublist == NULL) {
		g_list_append (seq->pieces, p); 
	} else {
		g_list_insert (sublist, p, 0);
	}
	seq->length += p->length;
	piece_ref (p);
}

static void 
sequence_remove_piece_at (sequence_t *seq, GList *sublist) {
	piece_t *p = (piece_t *) sublist->data;
	seq->pieces = g_list_remove_link (seq->pieces, sublist);
	g_list_free_1 (sublist);
	seq->length -= p->length;
	piece_free (p);
}

				
static GList *
node_containing (sequence_t *seq,
		 guint32 position, 
		 guint32 *piece_start)
	
{
	guint32 start_of_next;
	GList *listp;

	/* Avoid walking the list when we'll never find
	   the piece we want.
	*/

	if (position >= seq->length) {
		return NULL;
	}

	*piece_start = 0;
	start_of_next = 0;

	for (listp = seq->pieces; listp; listp = g_list_next (listp)) {
		start_of_next += ((piece_t *) listp->data)->length;
		if (start_of_next > position) {
			return listp;
		}
		*piece_start = start_of_next;
	}

	return NULL;
}

static void
sequence_split (sequence_t *seq, guint32 where, GList **before, GList **after)

{
	GList *listp;
	GList *insert_listp;
	piece_t *old;
	piece_t *earlier_piece;
	piece_t *later_piece;
	guint32  piece_start;
	guint32 offset;

	if (where == 0) {

		*before = NULL;
		*after = seq->pieces;

		return;

	} else if (where == seq->length) {

		*before = g_list_last (seq->pieces);
		*after = NULL;
		
		return;

	} else if (where > seq->length) {

		guint32 extra = where - seq->length;
		GList *last;

		if (silent_buffer == 0) {
			silent_buffer = const_buffer_new (0);
		}

		later_piece = piece_new (silent_buffer, 0, extra);
		last = g_list_last (seq->pieces);

		/* XXX this insertion doesn't show up in the state !!
		   
		   This is a special case of insert, and we inline it
		   because we have already just traversed the list
		   in order to find the last (about to be penultimate) 
		   node.
		*/
		
		g_list_insert (last, later_piece, 1);
		seq->length += later_piece->length;
		piece_ref (later_piece);
		
		*before = last;
		*after = g_list_next (last);

		return;
	}

	g_assert (where > 0 && where < seq->length);

	listp = node_containing (seq, where, &piece_start);

	if (listp == NULL) {
		g_error ("no node for position %u in length %u\n",
			 where, seq->length);
		abort();
	}

	if (piece_start == where) {
		
		*before = g_list_previous (listp);
		*after = listp;

		return;
	} 

	/* splice into the middle of a piece.
	*/
	
	offset = where - piece_start;
	old = (piece_t *) listp->data;
	
	earlier_piece = piece_new (old->buffer,
				   old->start,
				   offset);

	later_piece = piece_dup (old);
	later_piece->start += offset;
	later_piece->length -= offset;
	
	/* insert earlier piece,
	   store node that points to it,
	   insert later piece,
	   remove old piece,
	   store node that points to after the split,
	*/

	sequence_insert_piece_before (seq, listp, earlier_piece);
	*before = g_list_previous (listp);
	sequence_insert_piece_before (seq, listp, later_piece);
	*after = g_list_previous (listp);
	sequence_remove_piece_at (seq, listp);

	return;
}

static void
sequence_remove_range (sequence_t *seq, guint32 where, guint32 length, GList **after)

{
	GList *bs, *as, *be, *ae, *tmp;

	if (where >= seq->length) {
		g_warning ("EDL: deletion beyond sequence end; ignored\n");
		return;
	} else if (where + length > seq->length) {
		g_warning ("EDL: deletion extends beyond sequence end; truncated\n");
		length = seq->length - where;
	}

	sequence_split (seq, where, &bs, &as);
	sequence_split (seq, where+length, &be, &ae);

	/* `as' may point to a piece that is no longer valid, because it
	   had to be split to handle the end. check to see if this is
	   true.
	*/

	if (g_list_next (bs) != as) {

		/* OK, it was removed, so the "after start" piece
		   is actually the "before_end" piece.
		*/

		as = be;
	} 

	/* Now remove all the pieces between the start and the end */

	while (as != ae) {
		
		tmp = g_list_next (as);
		sequence_remove_piece_at (seq, as);
		as = tmp;
	}

	/* NOTE: *after may be NULL (i.e. we removed the entire end of
	   the sequence.
	*/

	*after = ae;
	return;
}

int
sequence_insert_sequence (sequence_t *seq, guint32 where, sequence_t *target)

{
	GList *not_used, *after;
	GList *s;

	if (seq->pieces == NULL) {
		seq->pieces = piece_list_copy (target->pieces);
		seq->length = target->length;
	} else {
		sequence_split (seq, where, &not_used, &after);
		
		for (s = target->pieces; s; s = g_list_next (s)) {
			piece_t *p = (piece_t *) s->data;
			sequence_insert_piece_before (seq, after, (piece_t *) s->data);

			/* XXX state should appear atomic w.r.t sequence insertion.
			 */

			sequence_save_state (seq, SequenceInsert, where, p->length, p, 0);
		}
	}

	return 0;
}

int
sequence_insert_piece (sequence_t *seq, guint32  where, piece_t *p)

{
	GList *not_used, *after;

	sequence_split (seq, where, &not_used, &after);
	sequence_insert_piece_before (seq, after, p);
	sequence_save_state (seq, SequenceInsert, where, p->length, p, 0);

	return 0;
}

int
sequence_delete (sequence_t *seq, guint32 start, guint32 length)

{
	GList *not_used;
	sequence_remove_range (seq, start, length, &not_used);
	sequence_save_state (seq, SequenceDelete, start, length, NULL, 0);
	return 0;
}

gint
sequence_replace_sequence (sequence_t *seq, guint32 start, sequence_t *rep)

{
	piece_t *piece;
	GList *listp;
	GList *after;

	sequence_remove_range (seq, start, rep->length, &after);

	for (listp = rep->pieces; listp; listp = g_list_next (listp)) {
		piece = (piece_t *) listp->data;

		sequence_insert_piece_before (seq, after, piece);
		
		/* XXX should be atomic w.r.t sequence insertion, but 
		   the API requires a piece-wise record. FIX ME.
		*/

		sequence_save_state (seq, SequenceReplace, start, piece->length, piece, 0);
	}

	return 0;
}

int
sequence_replace_piece (sequence_t *seq, guint32 start, piece_t *piece)

{
	GList *listp;
	GList *before, *after;

	sequence_remove_range (seq, start, piece->length, &after);
	sequence_insert_piece_before (seq, after, piece);
	sequence_merge_adjacent_pieces (seq);
	sequence_save_state (seq, SequenceReplace, start, piece->length, piece, 0);
//	sequence_dump (seq);

	return 0;
}

int
sequence_replace_data (sequence_t *seq, guint32 where, buffer_data_t *data, guint32 cnt) 
{
	piece_t *new_piece;

	if ((new_piece = seq->default_buffer->append_data (seq->default_buffer, data, cnt)) == NULL) {
		return -1;
	}

	sequence_replace_piece (seq, where, new_piece);
	return 0;
}

sequence_t *
sequence_subsequence (sequence_t *seq, guint32 start, guint32 cnt)

{
	sequence_t *new_seq;
	guint32 offset;
	GList  *node;
	piece_t *piece;
	piece_t *new_piece;
	guint32 this_copy;
	guint32 length;
	guint32 piece_start;

	/* Don't use sequence_split() we want to leave the original
	   sequence unchanged.
	*/

	if ((node = node_containing (seq, start, &piece_start)) == NULL) {
		return NULL;
	}

	new_seq = sequence_new (NULL, seq->default_buffer);
	
	while (cnt && node) {

		piece = (piece_t *) node->data;
		offset = start - piece_start;
		length = piece->length - offset;

		if (length > cnt) {
			length = cnt;
		}

		new_piece = piece_new (piece->buffer, piece_start+offset, length);
		sequence_insert_piece_before (new_seq, NULL, new_piece);
		
		node = g_list_next (node);
		start += length;
		piece_start += length;
		cnt -= length;
	}

	/* Note: subsequence may be shorter than requested */
	
	return new_seq;
}

static gint
sequence_move_to_add_file (sequence_t *seq)

{
	/* our job here: to create a new buffer containing
	   the entire contents of all the buffers referred
	   to by `seq', then to reset `seq' so that it uses
	   the new buffer rather than the old one(s). 

	   this allows us to do "destructive" operations
	   on a sequence and undo/redo them extremely quickly.
	*/

	GList *list, *listp;
	GList *buffer_list;
	piece_t *piece;
	buffer_t *new_buffer;

	list = seq->pieces;
	piece = (piece_t *) list->data;
	new_buffer = piece->buffer->clone (piece->buffer);

	for (list = seq->pieces; list; list = g_list_next (list)) {
		piece = (piece_t *) list->data;
		new_buffer->append (new_buffer, piece->buffer, piece->start, piece->length);
	}
		     
	/* Truncate the list down to just the very first piece.
	 */

	listp = list = g_list_remove_link (seq->pieces, seq->pieces);

	for (; listp; listp = g_list_next (listp)) {
		piece_free ((piece_t *) listp->data);
	}
	g_list_free (list);

	/* reset the first piece to use the new buffer, which now
	   contains the entire contents of this sequence in a
	   contiguous fashion.
	*/

	piece = (piece_t *) seq->pieces->data;

	buffer_free (piece->buffer);
	piece->buffer = buffer_ref (new_buffer);
	piece->start = 0;
	piece->length = seq->length;
	piece->refcnt = 1;

	return 0;
}

gint
sequence_apply (sequence_t *seq, guint32 start, guint32 cnt, 
		SequenceFunction function, gpointer arg)

{
	sequence_t *subseq;
	piece_t *piece;
	gint ret = 0;

	/* First, get a subsequence */

	if ((subseq = sequence_subsequence (seq, start, cnt)) == 0) {
		return -1;
	}

	/* Now move it into the add file (copies the data) */

	if (sequence_move_to_add_file (subseq)) {
		return -1;
	}

	/* Now there is only one piece. Apply the function
	   to it.
	*/

	piece = (piece_t *) subseq->pieces->data;
	
	if (function (piece->buffer, piece->start, piece->length, arg) != 0) {
		ret = -1;
		goto out;
	} 

	/* now replace the relevant section of the original sequence
	   with a piece that refers to the modified clone.
	*/

	sequence_replace_piece (seq, start, piece);

  out:
	sequence_free (subseq);
	return ret;
}

gint
sequence_state_stack_position (sequence_t *seq)

{
	return g_list_position (g_list_first (seq->state_ptr), seq->state_ptr);
}

gint
sequence_set_state_stack_position (sequence_t *seq, guint pos)

{
	sequence_state_t *state;

	if (pos >= g_list_length (seq->state_ptr)) {
		return -1;
	} 

	seq->state_ptr = g_list_nth (g_list_first (seq->state_ptr), pos);
	state = (sequence_state_t *) seq->state_ptr->data;

	seq->pieces = state->pieces;
	seq->length = seq->length;

	/* XXX send "modified" */
}

int
sequence_save (sequence_t *seq, const gchar *path)

{
	return sequence_store_as_edl (seq, path ? path : seq->path);
}

guint32 
sequence_read (sequence_t *seq, buffer_data_t *buf, guint32 start, guint32 cnt)

{
	guint32 copied;
	GList  *nth;
	guint32 this_copy;
	guint32 piece_start;
	piece_t *piece;
	guint32 offset;

	copied = 0;
	nth = node_containing (seq, start, &piece_start);

	/* note: nth may be NULL, in which case we will just zero fill
	   the entire request.
	*/

	while (nth && cnt) {
		piece = (piece_t *) nth->data;
		offset = start - piece_start;
		this_copy = piece->length - offset;
		if (this_copy > cnt) {
			this_copy = cnt;
		}

		if (piece->buffer->read (piece->buffer, buf+copied, piece->start+offset, this_copy) != this_copy) {
			return copied;
		}

		if (seq->gain != 1) {
			buffer_data_t *bp;
			size_t n;
			float g = seq->gain;

			for (n = 0, bp = buf+copied; n < this_copy; n++) {
				*bp = *bp * g;
				bp++;
			}
		}

		nth = g_list_next (nth);
		piece_start += piece->length;
		start += this_copy;
		cnt -= this_copy;
		copied += this_copy;
	}

	/* zero-fill anything that we couldn't get from the buffers */

	while (cnt--) {
		*buf++ = 0;
		copied++;
	}

	return copied;
}

void
sequence_set_gain (sequence_t *seq, float gain)

{
	seq->gain = gain;
	sequence_save_state (seq, SequenceSetGain, 0, seq->length, 0, gain);
}
